import numpy as np

# pristup elementima niza
a = np.array([1,2,3,4,5,6])
print(a)

# indeksni pristup elementima niza
print(a[0])
a[0] = 10
print(a)

# slicing - odsecanje
print(a[:3])

# pravimo novi "pogled" na niz
b = a[3:]
print(b)

# kada se radi o pajton listama, tada slicing kreira
# kopiju podniza. medjutim, kada se radi o ndarray
# nizovima, tada slicing samo kreira novi "pogled"
# na postojeci niz.

# posledica toga je da mozem da izmenimo polazni niz
# uz pomoc pogleda
b[0] = 200

# stampamo originalni niz da se uverimo da je izmenjen
print(a)

# ako nam treba duboka kopija postojeceg niza moramo da 
# koristimo metod copy
c = a.copy()
# menjamo kopiju polaznog niza
c[0] = -1000
print(c)
# polazni niz ostaje nepromenjen
print(a)

# kreiramo 3x4 niz
a = np.array([[1,2,3,4],[5,6,7,8],[9,10,11,12]])
print(a)

# pristup konkretnom elementu niza
a[1,3] = 200
print(a)

# indeksiranje

# pravimo matricu
a = np.array([[1,2,3,4],[5,6,7,8],[9,10,11,12]])

# izdvajamo sve elemente manje od 5
b = a[a < 5]
# rezultat indeksiranja je uvek niz, tj. skup elemenata polaznog
# n-dim niza koji ispunjavaju zadati uslov
print(b)

# uslove mozemo da pamtimo i kao odvojene promenljive
veci_od_5 = a > 5
# i zatim da ih iskoristmo da indeksiramo polazni niz
c = a[veci_od_5]
print(c)

# uslovi mogu biti i slozeni
veci_od_5_i_parni = (a%2==0) & (a > 5)
d = a[veci_od_5_i_parni]
print(d)

manji_od_5_ili_veci_od_10 = (a < 5) | (a > 10)
e = a[manji_od_5_ili_veci_od_10]
print(e)

# rezultat koji dobijemo definisanjem uslova je obicna bool matrica
print(manji_od_5_ili_veci_od_10)

# ako zelimo da izdvojimo indekse elemenata koji ispunjavaju dati uslov
# mozemo da koristimo metod np.nonzero()
indeksi = np.nonzero(a < 5)
# rezultat je lista indeksa po dimenzijama, tj. lista sacinjena od dva niza
print(indeksi)

# ako zelimo da dobijemo koordinate mozemo da ih slepimo u uredjeni par
indeksi = list(zip(indeksi[0], indeksi[1]))
# elementi niza indeksi su uredjeni parovi
print(indeksi)
# ako zelimo da obradimo elemente koji zadovoljavaju uslov to mozemo da 
# uradimo i petljom. 
for i in indeksi:
    print(a[i])
    
# filtriranje po uslovu je jedan nacin izdvajanja zeljenih elemenata, 
# ali cesto nam samo treba da eliminisemo duplikate iz niza
# to mozemo lako postici upotrebom metoda unique()

# kreiramo niz elemenata koji ima duplikate
a = np.array([11, 11, 12, 13, 14, 15, 16, 17, 12, 13, 11, 14, 18, 19, 20])
print("A : ", a)
# kreiramo niz bez duplikata
b = np.unique(a)
print("A bez duplikata: ", b)
# ako postavimo argument return_index na True, saznacemo i indekse prvih
# pojavljivanja elemenata u originalnom nizu
b, indeksi = np.unique(a, return_index=True)
print("Indeksi elemenata u polaznom nizu: ", indeksi)
# pored toga, mozemo i da saznamo koliko svaki pojedinacni element ima 
# duplikata u polaznom nizu. za to nam treba argument return_index
b, br_pojavljivanja = np.unique(a, return_counts=True)
print("Broj duplika u polaznom nizu: ", br_pojavljivanja)

# unique() se moze primeniti i visedimenzionalne nizove, pri cemu ce
# rezultat uvek biti jednodimanzioni niz

# kreiramo matricu
a = np.array([[1, 2, 3, 4], [5, 6, 7, 8], [9, 10, 11, 12], [1, 2, 3, 4]])
print("A: ", a, sep="\n")
# izdvajamo jedinstvene elemente iz matrice
b = np.unique(a)
print("Jedinstveni elemente", b)
# s obzirom da se radi o 2d nizu, metod se uopstava i omogucava nam da
# izdvajamo jedinstvene redove/kolone
redovi = np.unique(a, axis=0)
print("Jedinsveni redovi: ", redovi, sep="\n")
kolone = np.unique(a,axis=1)
print("Jedinsvene kolone: ", kolone, sep="\n")
# mozemo kombinovati i ranije uvedene atribute
redovi, indeksi, pojavljivanja = np.unique(a, axis=0, return_index=True, return_counts=True)
print("Jedinsveni redovi: ", redovi, sep="\n")
print("Indeksi: ", indeksi)
print("Pojavljivanja: ", pojavljivanja)
    
# pored filtriranja elemenata nizova, mozemo i da im menjamo redosled
a = np.arange(10)
print("A: ", a)
# ako zelimo da okrenemo niz kao u ogledalu treba da koristimo metod flip
b = np.flip(a)
print("A u ogledalu: ", b)

# isti metod se moze primeniti i na matrice
a = np.array([[1,2,3,4],[5,6,7,8],[9,10,11,12]])
print("Matrica: ", a, sep="\n")
b = np.flip(a)
# matrica u oglealu znaci da se prvo kao u ogledalu okrecu redovi,
# a zatim kolone
print("Matrica u ogledalu: ", b, sep="\n")
# medjutim, moguce je okrenuti samo jednu dimenziju matrice njenim 
# navodjenjem kao argumenta metoda flip
c = np.flip(a, axis = 0)
print("Redovi zamenjeni u ogledalu: ", c, sep="\n")
d = np.flip(a, axis = 1)
print("Kolone zamenjene u ogledalu: ", d, sep="\n")
# takodje, moguce je okretati kao u ogledalu i konkretne redove
a[1] = np.flip(a[1])
print("Prvi red okrenut kao u ogledalu: ", a, sep="\n")
# ali i kolone
a[:, 1] = np.flip(a[:,1])
print("Prva kolona okrenuta kao u ogledalu: ", a, sep="\n")