import numpy as np

# sortiranje, nadovezivanje, eliminisanje elemenata
# promena velicine/dimenzije niza, dodavanje novih dimenzija

# polazni, nesortirani niz
a = np.array([2,1,5,3,7,4,6,9])

# sortiranje niza (nije u mestu)
# podrazumevano, niz ce uvek biti sortiran neopadajuce
a = np.sort(a)

print(a)

# nadovezivanje nizova
a = np.array([1,2,3,4])
b = np.array([5,6,7,8])
c = np.concatenate((a,b))
print(c)

# nadovezivanje matrica (n-dim nizova u opstem slucaju)
a = np.array([[1,2],[3,4]])
b = np.array([[5,6]])
# pri nadovezivanju morate da vodite racuna da dimenzije 
# budu saglasne i da navedete dimenziju na koju zelite da nadovezete
# novi sadrzaj (parametar axis)

# nadovezujemo novi red
c = np.concatenate((a,b), axis=0)
print(c)

d = np.array([[1],[2],[3]])
# nadovezujemo novu kolonu
e = np.concatenate((c,d), axis=1)
print(e)

# pored nadovezivanja moguce je i elinisati sadrzaj iz n-dim nizova
# potrebno je da navedete index elementa zelite da izbacite
# i dimenziju o kojoj se radi

# eliminisemo prvi red
f = np.delete(e, 1, 0)
print(f)

# eliminisemo prvu kolonu
g = np.delete(f, 1, 1)
print(g)

# definisemo niz brojeva
# dimenzija niza je 1x6
a = np.arange(6)
print(a)

# prilikom promene oblika moramo da vodimo racuna
# da ukupan broj elemenata niza mora ostati isti
# i da zadate dimenzije delioci tog broja

# menjamo oblik niza u 2x3 matricu
# novi niz se popunjava red po red
b = np.reshape(a, newshape=(2,3))
print(b)

# novi niz se opet popunjava red po red
c = np.reshape(b, newshape=(6,1))
print(c)

# kreiramo novi niz, tj. vektor
a = np.arange(6)
print(a)
print(a.shape)

# mozemo da dodamo novu dimenziju ispred postojece, tj. da pretvorimo
# vektor u matricu. na ovaj nacin dobijamo 1x6 matricu
b = a[np.newaxis, :]
print(b.shape)

# ali mozemo da dodamo novu dimenziju i nakon postojece
# na taj nacin dobijamo, 6x1 matricu
c = a[:, np.newaxis]
print(c.shape)

# BITNO: dodavanje dimenzije ne znaci dodavanje podataka, vec samo 
#        povecavanje dimenzionalnosti naseg niza
# za dodavanje podataka mozemo da koristimo ranije uvedeno nadovezivanje 

# drugi nacin za povecavanje dimenzionalnosti je da koristimo
# expand_dims uz eksplicitno navodjenje dimenzije koju zelimo
# da dodamo

# pravimo 6x1 matricu
d = np.expand_dims(a, axis=1)
print(d.shape)

# pravimo (1x6) matricu
e = np.expand_dims(a, axis=0)
print(e.shape)

# dodavanje elemenata u postojece nizove mozemo da radimo pomocu
# funkcija vstack i hstack
a1 = np.array([[1,1],[2,2]])
a2 = np.array([[3,3],[4,4]])

# nadovezujemo matricu a2 pored matrice a1
b = np.hstack((a1,a2))
print(b)

# nadovezujemo matricu a2 ispod matrice a1
b = np.vstack((a1,a2))
print(b)

# pravimo novi niz 
a = np.arange(1,25)
print(a)

# pretvaramo ga u matricu
b = np.reshape(a, newshape=(2,12))
print(b)
# postojecu matricu mozemo da podelimo na vise nizova
# jednakih duzina duz horizontalne ose
b1,b2,b3 = np.hsplit(b, 3)
print(b1)
print(b2)
print(b3)

# pretvaramo ga u matricu
c = np.reshape(a, newshape=(12,2))
print(c)
# postojecu matricu mozemo da podelimo na vise nizova
# jednakih duzina duz vertikalne ose
c1,c2,c3 = np.vsplit(c, 3)
print(c1)
print(c2)
print(c3)


# pored povecanja dimenzije nizova, mozemo je i smanjiti
# takva operacija se zove ravnanje i u tome nam mogu pomoci dva 
# metoda
# flatten() - kreira kopiju originalnih podataka sa jednom dimenzijom manje
# ravel() - pravi novi pogled nad originalnim podacima

# kreiramo matricu
a = np.array([[1 , 2, 3, 4], [5, 6, 7, 8], [9, 10, 11, 12]])
print("A: ", a, sep="\n")
# ravnamo matricu u niz
b = a.flatten()
# menjamo kopiju
b[5]= 200
print("B: ", b, sep="\n")
# original ostaje nepromenjen
print("A: ", a, sep="\n")

# kreiramo novi izravnati pogled
c = a.ravel()
# menjamo pogled
c[5]= 200
print("C: ", c, sep="\n")
# cime je promenjen i original
print("A: ", a, sep="\n")