import numpy as np

# cuvanje i ucitavanje numpy podataka

# kreiramo niz
a = np.array([1,2,3,4,5,6])
print("Niz koji cuvamo u fajlu: ", a)

# cuvamo niz u numpy formatu (automatski se dodaje .npy ekstenzija)
# prilikom cuvanja, kodira se i format podataka
np.save("niz_a", a)
# ucitavamo niz iz fajla 
# nakon ucitavanja se rekonstruise polazni format niza
b = np.load("niz_a.npy")
# stampamo ucitani niz
print("Ucitani npy niz: ", b)

# cesto je zgodnije da podatke cuvamo u nekom formatriranom obliku koji je zgodan
# za kasniju obradu potencijalno drugacijim programom/paketom

# najcesce se koristi csv (comma separeted values) fromat
# u tom slucaju je neophodno da navedemo ekstenziju koju zelimo
np.savetxt("niz_a.csv", a)
# napomena: iako citljiviji i portabilniji, csv fajlovi su memorijski zahtevniji
# i neuporedivo sporiji za cuvanje/ucitavanje od npy fajlova. dodatno, prilikom
# cuvanja se ne kodira format podataka ako to eksplicitno ne navedemo, pa ce  
# brojevni podaci podrazumevano biti sacuvani u najsirem formatu, tj. kao realni

# podatke ucitavamo tako sto koristimo metod loadtxt()
# nakon ucitavanja, podaci ce biti realni brojevi
c = np.loadtxt("niz_a.csv")
print("Ucitani csv niz: ", c)


# savetxt metod omogucava i formatiranje samog fajla

# kreiramo matricu slucajnih brojeva
rng = np.random.default_rng()
# normalna raspodela sa m=1000 i std=200
# simulacija izmerenih podataka
a = rng.normal(1000, 200, size=(1000,5))

# zelimo da sacuvamo podatke u csv fajlu zaokruzene na dve decimale
# sa opisom kolona, tj. zaglavljem
header="x1, x2, x3, x4, x5"

# cuvamo csv fajl
np.savetxt("podaci.csv", a, fmt="%.2f", delimiter=",",header=header)

# ucitavamo podatke
# prilikom ucitavanja, moramo da odbacimo header
b = np.loadtxt("podaci.csv", dtype="float", delimiter=",", skiprows=1)
# stampamo prvih 10 ucitanih redova
print(b[0:10,:])