# NumPy biblioteka

NumPy biblioteka definiše strukture podataka za predstavljanje višedimenzionalnih nizova poput `ndarray` koja predstavlja N-dimenzionalne homogene nizove. Pored definicija struktura podataka, NumPy biblioteka definiše i veliki broj funkcija koje se mogu efikasno izvršavati nad njima.

Da bismo mogli da koristimo NumPy biblioteku postoji nekoliko preduslova:

1. Instaliran Python 3.12 ili noviji
2. Instaliran i pravilno konfigurisan `pip` menadžer paketa.

Biblioteka NumPy se jednostavno instalira na Widnows operativnim sistemima izvršavanjem komande u terminalu:

`python -m pip install numpy`

dok je na Linux operativnim sistemima neophodno izvršiti komandu

`pip install numpy`

Po završetku instalacije, lako se proverava da li je instalacija bila uspešna pomoću sledećeg skripta:

~~~py
import numpy as np

print(np.__version__)
~~~

U slučaju da se prilikom izvršavanja skripta desi greška, znači da instalacija nije bila uspešna i da treba da ponovite postupak.

## Prednosti NumPy biblioteke

Klasične liste u Pajtonu su jednostavne i izuzetno fleksibilne strukture podataka koje nam omogućavaju da čuvamo heterogene podatke, tj. podatke različitih tipova i/ili dimenzija, u istom kontejneru. Pored toga, prilično su efikasne kada treba da primenimo željene operacije na pojedinačne elemente u listi.

Međutim, u zavisnosti od tipova podataka, ali i od operacija koje želimo da primenimo, može se desiti da su neke druge strukture podataka pogodnije. Iskorišćavanjem znanja o tipovima podataka i željenim operacijama može se dobiti značajno povećanje perfomansi. NumPy biblioteka se koristi upravo u takvim slučajevima, jer značajno ubrzava izvršavanje operacija nad velikim količinima homogenih podataka, tj. podataka istog tipa. 

## NumPy nizovi

Niz je linearna kolekcija podataka istog tipa koja je neprekidna u memoriji. Svakoj ćeliji niza dodeljen je jedan element. Dvodimenzionalni niz dobićemo slaganjem više jednodimenzionalnih nizova jedan na drugi. Na isti način možemo dobiti trodimenzionalni niz slaganjem više dvodimenzionalnih nizova jedan na drugi itd.

Uopštavanjem ove ideje na proizvoljni broj dimenzija dobijamo N-dimenzionalne nizove. Zbog toga, osnovna klasa kojom su predstavljeni NumPy nizovi se naziva `ndarray` i predstavlja N-dimenzionalni niz. U nastavku teksta termin niz će označavati instancu klase `ndarray`

## NumPy ograničenja

Da bi se osigurala efikasnost operacija sa N-dimenzionalnim nizovima, NumPy biblioteka postavlja nekoliko ograničenja nad nizovima:

1. Svi elementi niza moraju biti istog tipa.
2. Nakon kreiranja, ukupan broj elemenata niza se ne može promeniti.
3. Oblik niza mora biti *pravougaoni*, a ne *stepenasti*. Na primer, svaki red u 2D nizu mora imati jednak broj elemenata, tj. kolona.

Ukoliko su ispunjena sva ograničenja, tada će NumPy biblioteka iskoristiti karakteristike niza da bi sve operacije nad nizovima učinila i vremenski i memorijski efikasnijim. Na ovaj način dobija se intuitivniji i jednostavniji interfejs za korišćenje nizova u poređenju sa manje restriktivnim strukturama podataka. 

## Terminologija

U matematici često imamo sledeće termine:

1. 0D niz se naziva skalar.
2. 1D niz se naziva vektor.
3. 2D niz se naziva matrica.
4. ND niz se naziva tenzor. (N > 2)

Međutim, u radu sa NumPy nizovima treba izbegavati matematičke termine, jer se pomenuti matematički objekti ponašaju značajno drugačije od jednostavnih N-dimenzionalnih nizova. Na primer, množenje matrica je suštinski drugačija operacija od *množenja* nizova. Dodatni razlog zbog kojeg treba da izbegavamo matematičku interpretaciju N-dimenzionalnih nizova je činjenica da u sklopu drugih pajton paketa postoje strukture podataka sa baš tim nazivima.